@@ -158,15 +158,35 @@ Java_org_krita_android_JNIWrappers_saveState(JNIEnv* /*env*/,
     kritarc.setValue("canvasState", "OPENGL_SUCCESS");
 }
 
-extern "C" JNIEXPORT void JNICALL
+extern "C" JNIEXPORT jboolean JNICALL
 Java_org_krita_android_JNIWrappers_exitFullScreen(JNIEnv* /*env*/,
                                                   jobject /*obj*/,
                                                   jint    /*n*/)
 {
-    if (!KisPart::exists()) return;
+    if (!KisPart::exists()) {
+        return false;
+    }
 
     KisMainWindow *mainWindow = KisPart::instance()->currentMainwindow();
-    mainWindow->viewFullscreen(false);
+    if (mainWindow) {
+        mainWindow->viewFullscreen(false);
+        return true;
+    } else {
+        return false;
+    }
+}
+
+extern "C" JNIEXPORT void JNICALL
+Java_org_krita_android_JNIWrappers_openFileFromIntent(JNIEnv* /*env*/,
+                                                      jobject /*obj*/,
+                                                      jstring str)
+{
+    QAndroidJniObject jUri(str);
+    if (jUri.isValid()) {
+        QString uri = jUri.toString();
+        QMetaObject::invokeMethod(KisApplication::instance(), "fileOpenRequested",
+                                  Qt::QueuedConnection, Q_ARG(QString, uri));
+    }
 }
 
 __attribute__ ((visibility ("default")))
@@ -448,6 +468,12 @@ extern "C" int main(int argc, char **argv)
 
     // first create the application so we can create a pixmap
     KisApplication app(key, argc, argv);
+
+    if (app.platformName() == "wayland") {
+        QMessageBox::critical(0, i18nc("@title:window", "Fatal Error"), i18n("Krita does not support the Wayland platform. Use XWayland to run Krita on Wayland. Krita will close now."));
+        return -1;
+    }
+
     KisUsageLogger::writeHeader();
     KisOpenGL::initialize();
 
