@@ -19,6 +19,8 @@
 #include "kis_updater_context_test.h"
 #include <QTest>
 
+#include "kistest.h"
+
 #include <QAtomicInt>
 #include <KoColorSpace.h>
 #include <KoColorSpaceRegistry.h>
@@ -32,7 +34,7 @@
 #include "scheduler_utils.h"
 
 #include "lod_override.h"
-
+#include "config-limit-long-tests.h"
 
 void KisUpdaterContextTest::testJobInterference()
 {
@@ -45,7 +47,7 @@ void KisUpdaterContextTest::testJobInterference()
 
     KisPaintLayerSP paintLayer = new KisPaintLayer(image, "test", OPACITY_OPAQUE_U8);
 
-    image->lock();
+    image->barrierLock();
     image->addNode(paintLayer);
     image->unlock();
 
@@ -166,7 +168,11 @@ void KisUpdaterContextTest::testSnapshot()
 }
 
 #define NUM_THREADS 10
-#define NUM_JOBS 6000
+#ifdef LIMIT_LONG_TESTS
+#   define NUM_JOBS 60
+#else
+#   define NUM_JOBS 6000
+#endif
 #define EXCLUSIVE_NTH 3
 #define NUM_CHECKS 10
 #define CHECK_DELAY 3 // ms
@@ -199,6 +205,10 @@ public:
         m_counter.deref();
     }
 
+    QString debugId() const override {
+        return "ExclusivenessCheckerStrategy";
+    }
+
 private:
     QAtomicInt &m_counter;
     QAtomicInt &m_hadConcurrency;
@@ -237,5 +247,5 @@ void KisUpdaterContextTest::stressTestExclusiveJobs()
              << "/" << NUM_CHECKS * NUM_JOBS;
 }
 
-QTEST_MAIN(KisUpdaterContextTest)
+KISTEST_MAIN(KisUpdaterContextTest)
 
