@@ -264,10 +264,7 @@ bool saveResourceToStore(KoResource *resource, KoStore *store, const QString &re
         warnKrita << "Could not open buffer";
         return false;
     }
-    if(!store->hasFile(resType + "/" + resource->shortFilename())) {
-        warnKrita << "Store does not have file" << (resType + "/" + resource->shortFilename());
-        return false;
-    }
+    Q_ASSERT(!store->hasFile(resType + "/" + resource->shortFilename()));
     if (!store->open(resType + "/" + resource->shortFilename())) {
         warnKrita << "Could not open file in store for resource";
         return false;
@@ -400,6 +397,29 @@ bool KisResourceBundle::save()
                 }
             }
         }
+#if defined HAVE_SEEXPR
+        else if (resType == "kis_seexpr_scripts")
+        {
+            KoResourceServer<KisSeExprScript> *seExprScriptServer = KoResourceServerProvider::instance()->seExprScriptServer();
+            Q_FOREACH (const KisResourceBundleManifest::ResourceReference &ref, m_manifest.files(resType))
+            {
+                KoResource *res = seExprScriptServer->resourceByMD5(ref.md5sum);
+                if (!res)
+                    res = seExprScriptServer->resourceByFilename(QFileInfo(ref.resourcePath).fileName());
+                if (!saveResourceToStore(res, store.data(), "seexpr_scripts"))
+                {
+                    if (res)
+                    {
+                        warnKrita << "Could not save resource" << resType << res->name();
+                    }
+                    else
+                    {
+                        warnKrita << "could not find resource for" << QFileInfo(ref.resourcePath).fileName();
+                    }
+                }
+            }
+        }
+#endif
     }
 
     if (!m_thumbnail.isNull()) {
@@ -448,7 +468,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KoAbstractGradient *res = gradientServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                QScopedPointer<KoAbstractGradient> res(gradientServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
                     continue;
@@ -465,7 +485,7 @@ bool KisResourceBundle::install()
 
                 KoAbstractGradient *res2 = gradientServer->resourceByName(res->name());
                 if (!res2)  {//if it doesn't exist...
-                    gradientServer->addResource(res, false);//add it!
+                    gradientServer->addResource(res.data(), false);//add it!
 
                     if (!m_gradientsMd5Installed.contains(res->md5())) {
                         m_gradientsMd5Installed.append(res->md5());
@@ -475,9 +495,11 @@ bool KisResourceBundle::install()
                     }
 
                     Q_FOREACH (const QString &tag, ref.tagList) {
-                        gradientServer->addTag(res, tag);
+                        gradientServer->addTag(res.data(), tag);
                     }
-                    //gradientServer->addTag(res, name());
+
+                    // release resource from the scoped pointera
+                    res.take();
                 }
                 else {
                     //warnKrita << "Didn't install" << res->name()<<"It already exists on the server";
@@ -492,7 +514,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KoPattern *res = patternServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                QScopedPointer<KoPattern> res(patternServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
                     continue;
@@ -509,7 +531,7 @@ bool KisResourceBundle::install()
 
                 KoPattern *res2 = patternServer->resourceByName(res->name());
                 if (!res2)  {//if it doesn't exist...
-                    patternServer->addResource(res, false);//add it!
+                    patternServer->addResource(res.data(), false);//add it!
 
                     if (!m_patternsMd5Installed.contains(res->md5())) {
                         m_patternsMd5Installed.append(res->md5());
@@ -519,9 +541,11 @@ bool KisResourceBundle::install()
                     }
 
                     Q_FOREACH (const QString &tag, ref.tagList) {
-                        patternServer->addTag(res, tag);
+                        patternServer->addTag(res.data(), tag);
                     }
-                    //patternServer->addTag(res, name());
+
+                    // release resource from the scoped pointera
+                    res.take();
                 }
 
             }
@@ -533,7 +557,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KisBrushSP res = brushServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                KisBrushSP res = brushServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath));
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
                     continue;
@@ -585,7 +609,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KoColorSet *res = paletteServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                QScopedPointer<KoColorSet> res(paletteServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
 
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
@@ -604,7 +628,7 @@ bool KisResourceBundle::install()
                 //find the resource on the server
                 KoColorSet *res2 = paletteServer->resourceByName(res->name());
                 if (!res2)  {//if it doesn't exist...
-                    paletteServer->addResource(res, false);//add it!
+                    paletteServer->addResource(res.data(), false);//add it!
 
                     if (!m_palettesMd5Installed.contains(res->md5())) {
                         m_palettesMd5Installed.append(res->md5());
@@ -614,9 +638,11 @@ bool KisResourceBundle::install()
                     }
 
                     Q_FOREACH (const QString &tag, ref.tagList) {
-                        paletteServer->addTag(res, tag);
+                        paletteServer->addTag(res.data(), tag);
                     }
-                    //paletteServer->addTag(res, name());
+
+                    // release resource from the scoped pointera
+                    res.take();
                 }
                 else {
                     //warnKrita << "Didn't install" << res->name()<<"It already exists on the server";
@@ -630,7 +656,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KisWorkspaceResource *res = workspaceServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                QScopedPointer<KisWorkspaceResource> res(workspaceServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
                     continue;
@@ -648,7 +674,7 @@ bool KisResourceBundle::install()
                 //the following tries to find the resource by name.
                 KisWorkspaceResource *res2 = workspaceServer->resourceByName(res->name());
                 if (!res2)  {//if it doesn't exist...
-                    workspaceServer->addResource(res, false);//add it!
+                    workspaceServer->addResource(res.data(), false);//add it!
 
                     if (!m_workspacesMd5Installed.contains(res->md5())) {
                         m_workspacesMd5Installed.append(res->md5());
@@ -658,9 +684,11 @@ bool KisResourceBundle::install()
                     }
 
                     Q_FOREACH (const QString &tag, ref.tagList) {
-                        workspaceServer->addTag(res, tag);
+                        workspaceServer->addTag(res.data(), tag);
                     }
-                    //workspaceServer->addTag(res, name());
+
+                    // release resource from the scoped pointera
+                    res.take();
                 }
                 else {
                     //warnKrita << "Didn't install" << res->name()<<"It already exists on the server";
@@ -675,7 +703,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KisPaintOpPresetSP res = paintoppresetServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                KisPaintOpPresetSP res = paintoppresetServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath));
 
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
@@ -724,7 +752,7 @@ bool KisResourceBundle::install()
                 if (resourceStore->isOpen()) resourceStore->close();
 
                 dbgResources << "\tInstalling" << ref.resourcePath;
-                KoGamutMask *res = gamutMaskServer->createResource(QString("bundle://%1:%2").arg(filename()).arg(ref.resourcePath));
+                QScopedPointer<KoGamutMask> res(gamutMaskServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
 
                 if (!res) {
                     warnKrita << "Could not create resource for" << ref.resourcePath;
@@ -743,7 +771,7 @@ bool KisResourceBundle::install()
                 //find the resource on the server
                 KoGamutMask *res2 = gamutMaskServer->resourceByName(res->name());
                 if (!res2)  {//if it doesn't exist...
-                    gamutMaskServer->addResource(res, false);//add it!
+                    gamutMaskServer->addResource(res.data(), false);//add it!
 
                     if (!m_gamutMasksMd5Installed.contains(res->md5())) {
                         m_gamutMasksMd5Installed.append(res->md5());
@@ -753,15 +781,78 @@ bool KisResourceBundle::install()
                     }
 
                     Q_FOREACH (const QString &tag, ref.tagList) {
-                        gamutMaskServer->addTag(res, tag);
+                        gamutMaskServer->addTag(res.data(), tag);
                     }
-                    //gamutMaskServer->addTag(res, name());
+
+                    // release resource from the scoped pointera
+                    res.take();
                 }
                 else {
                     //warnKrita << "Didn't install" << res->name()<<"It already exists on the server";
                 }
             }
         }
+#if defined HAVE_SEEXPR
+        else if (resType == "seexpr_scripts")
+        {
+            KoResourceServer<KisSeExprScript> *seExprScriptServer = KoResourceServerProvider::instance()->seExprScriptServer();
+            Q_FOREACH (const KisResourceBundleManifest::ResourceReference &ref, m_manifest.files(resType))
+            {
+
+                if (resourceStore->isOpen())
+                    resourceStore->close();
+
+                dbgResources << "\tInstalling" << ref.resourcePath;
+                QScopedPointer<KisSeExprScript> res(seExprScriptServer->createResource(QString("bundle://%1:%2").arg(filename(), ref.resourcePath)));
+
+                if (!res)
+                {
+                    warnKrita << "Could not create resource for" << ref.resourcePath;
+                    continue;
+                }
+                if (!resourceStore->open(ref.resourcePath))
+                {
+                    warnKrita << "Failed to open" << ref.resourcePath << "from bundle" << filename();
+                    continue;
+                }
+                if (!res->loadFromDevice(resourceStore->device()))
+                {
+                    warnKrita << "Failed to load" << ref.resourcePath << "from bundle" << filename();
+                    continue;
+                }
+                dbgResources << "\t\tresource:" << res->name();
+
+                //find the resource on the server
+                KisSeExprScript *res2 = seExprScriptServer->resourceByName(res->name());
+                if (!res2)
+                {
+                    //if it doesn't exist...
+                    seExprScriptServer->addResource(res.data(), false); //add it!
+
+                    if (!m_seExprScriptsMd5Installed.contains(res->md5()))
+                    {
+                        m_seExprScriptsMd5Installed.append(res->md5());
+                    }
+                    if (ref.md5sum != res->md5())
+                    {
+                        md5Mismatch.append(res->name());
+                    }
+
+                    Q_FOREACH (const QString &tag, ref.tagList)
+                    {
+                        seExprScriptServer->addTag(res.data(), tag);
+                    }
+
+                    // release resource from the scoped pointera
+                    res.take();
+                }
+                else
+                {
+                    //warnKrita << "Didn't install" << res->name()<<"It already exists on the server";
+                }
+            }
+        }
+#endif
     }
     m_installed = true;
     if(!md5Mismatch.isEmpty()){
@@ -848,6 +939,19 @@ bool KisResourceBundle::uninstall()
         }
     }
 
+#if defined HAVE_SEEXPR
+    KoResourceServer<KisSeExprScript> *seExprScriptServer = KoResourceServerProvider::instance()->seExprScriptServer();
+    //Q_FOREACH (const KisResourceBundleManifest::ResourceReference &ref, m_manifest.files("seexpr_scripts")) {
+    Q_FOREACH (const QByteArray md5, m_seExprScriptsMd5Installed)
+    {
+        KisSeExprScript *res = seExprScriptServer->resourceByMD5(md5);
+        if (res)
+        {
+            seExprScriptServer->removeResourceFromServer(res);
+        }
+    }
+#endif
+
     Q_FOREACH(const QString &tag, tags) {
         paintoppresetServer->tagCategoryRemoved(tag);
         workspaceServer->tagCategoryRemoved(tag);
@@ -940,6 +1044,13 @@ QList<KoResource*> KisResourceBundle::resources(const QString &resType) const
             KoResource *res =  gamutMaskServer->resourceByMD5(ref.md5sum);
             if (res) ret << res;
         }
+#if defined HAVE_SEEXPR
+        else if (resType == "seexpr_scripts") {
+            KoResourceServer<KisSeExprScript> *seExprScriptServer = KoResourceServerProvider::instance()->seExprScriptServer();
+            KoResource *res = seExprScriptServer->resourceByMD5(ref.md5sum);
+            if (res) ret << res;
+        }
+#endif
     }
     return ret;
 }
