@@ -28,14 +28,19 @@
 #include <QVBoxLayout>
 #include <QHBoxLayout>
 #include <QPushButton>
+#include <QDate>
 #include <QApplication>
 #include <QFile>
 #include <QDesktopServices>
+#include <kaboutdata.h>
 
 #include <klocalizedstring.h>
 
 #include "../../krita/data/splash/splash_screen.xpm"
+#include "../../krita/data/splash/splash_holidays.xpm"
 #include "../../krita/data/splash/splash_screen_x2.xpm"
+#include "../../krita/data/splash/splash_holidays_x2.xpm"
+
 #include "kis_splash_screen.h"
 
 KisAboutApplication::KisAboutApplication(QWidget *parent)
@@ -47,8 +52,17 @@ KisAboutApplication::KisAboutApplication(QWidget *parent)
     vlayout->setMargin(0);
     QTabWidget *wdgTab = new QTabWidget;
     vlayout->addWidget(wdgTab);
+    KisSplashScreen *splash = 0;
+
+    QDate currentDate = QDate::currentDate();
+    if (currentDate > QDate(currentDate.year(), 12, 4) ||
+            currentDate < QDate(currentDate.year(), 1, 9)) {
+        splash = new KisSplashScreen(qApp->applicationVersion(), QPixmap(splash_holidays_xpm), QPixmap(splash_holidays_x2_xpm));
+    }
+    else {
+        splash = new KisSplashScreen(qApp->applicationVersion(), QPixmap(splash_screen_xpm), QPixmap(splash_screen_x2_xpm));
+    }
 
-    KisSplashScreen *splash = new KisSplashScreen(qApp->applicationVersion(), QPixmap(splash_screen_xpm), QPixmap(splash_screen_x2_xpm), true);
     splash->setWindowFlags(Qt::Widget);
     splash->displayLinks(true);
     splash->setFixedSize(splash->sizeHint());
@@ -77,7 +91,22 @@ KisAboutApplication::KisAboutApplication(QWidget *parent)
     authors.chop(2);
     authors.append(".</p></body></html>");
     lblAuthors->setText(authors);
-    wdgTab->addTab(lblAuthors, i18n("Authors"));
+    wdgTab->addTab(lblAuthors, i18nc("Heading for the list of Krita authors/developers", "Authors"));
+
+
+    // Translators
+    KAboutData aboutData(KAboutData::applicationData());
+    if (aboutData.translators().isEmpty()) {
+        aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"),
+                                i18nc("EMAIL OF TRANSLATORS", "Your emails"));
+
+    }
+
+    if (!aboutData.translators().isEmpty()) {
+        wdgTab->addTab(createTranslatorsWidget(aboutData.translators(), aboutData.ocsProviderUrl()),
+                       i18nc("@title:tab", "Translators"));
+    }
+
 
     QTextEdit *lblKickstarter = new QTextEdit();
     lblKickstarter->setReadOnly(true);
@@ -90,11 +119,10 @@ KisAboutApplication::KisAboutApplication(QWidget *parent)
 
     QFile fileBackers(":/backers.txt");
     Q_ASSERT(fileBackers.exists());
-    fileBackers.open(QIODevice::ReadOnly);
-    Q_FOREACH (const QByteArray &backer, fileBackers.readAll().split('\n')) {
-        backers.append(QString::fromUtf8(backer));
-        backers.append(", ");
-    }
+    fileBackers.open(QIODevice::ReadOnly | QIODevice::Text);
+    QTextStream backersText(&fileBackers);
+    backersText.setCodec("UTF-8");
+    backers.append(backersText.readAll().replace("\n", ", "));
     backers.chop(2);
     backers.append(i18n(".</p><p><i>Thanks! You were all <b>awesome</b>!</i></p></body></html>"));
     lblKickstarter->setText(backers);
@@ -188,6 +216,7 @@ KisAboutApplication::KisAboutApplication(QWidget *parent)
 
 
     QPushButton *bnClose = new QPushButton(i18n("Close"));
+    bnClose->setIcon(QIcon::fromTheme(QStringLiteral("dialog-close")));
     connect(bnClose, SIGNAL(clicked()), SLOT(close()));
 
     QHBoxLayout *hlayout = new QHBoxLayout;
@@ -195,8 +224,40 @@ KisAboutApplication::KisAboutApplication(QWidget *parent)
     hlayout->addStretch(10);
     hlayout->addWidget(bnClose);
 
-
-
     vlayout->addLayout(hlayout);
 
 }
+
+QWidget *KisAboutApplication::createTranslatorsWidget(const QList<KAboutPerson> &translators, const QString &ocsProviderUrl)
+{
+    QString aboutTranslationTeam = KAboutData::aboutTranslationTeam();
+
+    qDebug() << aboutTranslationTeam << ocsProviderUrl;
+
+    QTextBrowser *lblTranslators = new QTextBrowser();
+
+    lblTranslators->setOpenExternalLinks(true);
+
+    QString translatorHtml = i18n("<html>"
+                                  "<head/>"
+                                  "<body>"
+                                  "<h1 align=\"center\"><b>Translators</b></h1>"
+                                  "<p><ul>");
+
+
+    Q_FOREACH(const KAboutPerson &person, translators) {
+        translatorHtml.append(QString("<li>%1</li>").arg(person.name()));
+    }
+
+    translatorHtml.append("<ul></p>");
+    translatorHtml.append(i18n("<p>KDE is translated into many languages thanks to the work of the "
+                          "translation teams all over the world.</p><p>For more information on KDE "
+                          "internationalization visit <a href=\"http://l10n.kde.org\">http://l10n."
+                          "kde.org</a></p>"));
+    translatorHtml.append("</body></html>");
+
+    lblTranslators->setText(translatorHtml);
+
+    return lblTranslators;
+}
+
