@@ -37,7 +37,6 @@ public:
     void resetRotationCenterButtons();
     void setDefaultWarpPoints(int pointsPerLine = -1);
     void setTooBigLabelVisible(bool value);
-    bool showDecorations() const;
     bool workRecursively() const;
 
 public Q_SLOTS:
@@ -47,7 +46,8 @@ public Q_SLOTS:
 Q_SIGNALS:
     void sigConfigChanged();
     void sigApplyTransform();
-    void sigResetTransform();
+    void sigResetTransform(ToolTransformArgs::TransformMode mode);
+    void sigCancelTransform();
     void sigRestartTransform();
     void sigEditingFinished();
 
@@ -97,6 +97,9 @@ public Q_SLOTS:
     void slotSetLiquifyModeButtonClicked(bool);
     void slotButtonBoxClicked(QAbstractButton *button);
 
+    void slotSetMeshModeButtonClicked(bool);
+
+
     void slotEditCagePoints(bool value);
 
     void liquifySizeChanged(qreal value);
@@ -112,11 +115,14 @@ public Q_SLOTS:
 
     void notifyEditingFinished();
 
+    void slotGranularityChanged(QString value);
+    void slotPreviewGranularityChanged(QString value);
+
+    void slotMeshSizeChanged();
+    void slotMeshShowHandlesChanged();
+    void slotMeshSymmetricalHandlesChanged();
+
 private:
-    // rad being in |R, the returned value is in [0; 360]
-    double radianToDegree(double rad);
-    // degree being in |R, the returned value is in [0; 2*M_PI]
-    double degreeToRadian(double degree);
 
     void blockNotifications();
     void unblockNotifications();
